
NODEFAULT: str = "REQUIRED: NO_DEFAULT"
DEFAULT_SYNC_CURRENT_FIRST: bool = True
DEFAULT_BILL_MANAGMENT: bool = False

DEFAULT_CLI_HOST: str = '127.0.0.1'
DEFAULT_CLI_PORT: int = 8990
DEFAULT_CLI_TIMEOUT: int = 30
DEFAULT_CLI_AUTH: bool = True
DEFAULT_CLI_USERNAME: str = "jcliadmin"
DEFAULT_CLI_PASSWORD: str = "jclipwd"
DEFAULT_CLI_STANDARD_PROMPT: str = "jcli : "
DEFAULT_CLI_INTERACTIVE_PROMPT: str = "> "

DEFAULT_LOG_PATH: str = "/var/log/jasmin/"
DEFAULT_LOG_LEVEL: str = "INFO"



import logging

import pkg_resources
from jasmin_telnet.proxy import Proxy as JasminTelnetProxy
from pymongo import MongoClient
from pymongo import errors as MongoErrors
from pymongo.database import Database as MongoDatabase

from jasmin_mongo_configuration.defaults import *


class MongoDB:

    def __init__(self, connection_string, database_name):
        """ Constructor """
        self.connection_string = connection_string
        self.database_name = database_name
        self.cli_conn: dict = {}
        logging.info("Starting MongoDB cluster's connection")

    def logger_callback(self, msg: str):
        logging.info(msg=msg)

    def startConnection(self) -> bool:
        mongoclient = MongoClient(self.connection_string)
        server_info = mongoclient.server_info()
        if isinstance(server_info, dict) and 'ok' in server_info and server_info['ok'] == 1:
            logging.info("Connected to MongoDB")
            self.mongoclient = mongoclient
            adminDatabase: MongoDatabase = self.mongoclient['admin']
            database: MongoDatabase = self.mongoclient[self.database_name]
            adminDatabase_info = adminDatabase.command("buildinfo")
            database_info = database.command("buildinfo")
            if isinstance(adminDatabase_info, dict) and isinstance(database_info, dict) and \
                    'ok' in adminDatabase_info and 'ok' in database_info and \
                    adminDatabase_info['ok'] == 1 and database_info['ok'] == 1:
                clusterParameter = adminDatabase.command(
                    {'getClusterParameter': "changeStreamOptions"})
                preAndPostImagesExpireAfterSeconds = clusterParameter[
                    'clusterParameters'][0]['preAndPostImages']['expireAfterSeconds']
                if isinstance(preAndPostImagesExpireAfterSeconds, int) is False or preAndPostImagesExpireAfterSeconds == 0:
                    logging.info("")
                    logging.info("Detected Pre and Post Images is not active")
                    adminDatabase.command({
                        'setClusterParameter':
                        {'changeStreamOptions': {
                            'preAndPostImages': {'expireAfterSeconds': 100}}}
                    })
                    logging.info(
                        "Cluster Parameters changed to support Pre and Post Images for 100 seconds")
                else:
                    logging.info("")
                    logging.info(
                        f"Cluster Parameters support Pre and Post Images for {preAndPostImagesExpireAfterSeconds} seconds")

                logging.info("")

                logging.info(f"Set to use database: {self.database_name}")
                logging.info("")
                self.adminDatabase = adminDatabase
                self.database = database
                return True
            else:
                logging.critical(
                    f"Failed to use database: {self.database_name}")
                return False
        else:
            logging.critical("Failed to connect to MongoDB")
            return False

    def set_bill_managment_state(self, state: bool):
        self.bill_managment: bool = state

    def get_bill_managment_state(self) -> bool:
        return hasattr(self, "bill_managment") and isinstance(self.bill_managment, bool) and self.bill_managment is True

    def get_one_module(self, module: str) -> dict[str, str | float | bool]:
        "" ""
        data: dict[str, str | float | bool] = {}
        cursor = self.database[module].find()
        for row in cursor:
            sub_id = row["_id"]
            del row["_id"]
            data[sub_id] = row

        return data

    def get_one_submodule(self, module: str, sub_id: str) -> dict[str, str | float | bool]:
        "" ""
        return self.database[module].find_one({"_id": sub_id})

    def insert_one(self, module, sub_id, data):
        data["_id"] = sub_id
        self.database[module].insert_one(data)

    def increment_one(self, module, sub_id, data):
        self.database[module].update_one(
            {"_id": sub_id}, {'$inc': data})

    def update_one(self, module, sub_id, data, upsert=True):
        self.database[module].update_one(
            {"_id": sub_id}, {'$set': data}, upsert=upsert)

    def pullAllConfigurations(self):
        "" ""
        modules_to_sync = ["smppccm", "httpccm", "group", "user", "filter",
                           "mointerceptor", "mtinterceptor", "morouter", "mtrouter"]

        logging.info(
            "Synchronizing all configuration to Jasmin from MongoDB cluster. Applies to:")
        for module in modules_to_sync:
            sub_data: dict = {}
            cursor = self.database[module].find()

            for row in cursor:
                if self.get_bill_managment_state() is True:
                    if module == "user":
                        row["mt_messaging_cred quota balance"] = "None"
                        row["mt_messaging_cred quota sms_count"] = "None"

                sub_id = row["_id"]
                del row["_id"]
                sub_data[sub_id] = row

            self.stream_handler(
                payload={
                    "change_type": "sync",
                    "module": module,
                    "sub_data": sub_data
                })

        logging.info("Finished applying configurations")

    def stream(
        self,
        cli_host: str = DEFAULT_CLI_HOST,
        cli_port: int = DEFAULT_CLI_PORT,
        cli_timeout: int = DEFAULT_CLI_TIMEOUT,
        cli_auth: bool = DEFAULT_CLI_AUTH,
        cli_username: str = DEFAULT_CLI_USERNAME,
        cli_password: str = DEFAULT_CLI_PASSWORD,
        cli_standard_prompt: str = DEFAULT_CLI_STANDARD_PROMPT,
        cli_interactive_prompt: str = DEFAULT_CLI_INTERACTIVE_PROMPT,
        syncCurrentFirst: bool = False
    ):

        self.cli_conn: dict = {
            "host": cli_host,
            "port": cli_port,
            "timeout": cli_timeout,
            "auth": cli_auth,
            "username": cli_username,
            "password": cli_password,
            "standard_prompt": cli_standard_prompt,
            "interactive_prompt": cli_interactive_prompt
        }

        try:
            with self.database.watch(full_document='updateLookup') as stream:
                if self.get_bill_managment_state() is True:
                    logging.info("Bill managment Enabled!")
                    logging.warn("Need to install interceptors manually!")
                    logging.info(
                        "For the MT Interceptor blueprint, please use the following command:")
                    logging.info(
                        "jasminmongoconfd -get-interceptor mt > mtinterceptor.py")
                    logging.info(
                        "For the MO Interceptor blueprint, please use the following command:")
                    logging.info(
                        "jasminmongoconfd -get-interceptor mo > mointerceptor.py")
                    logging.info(
                        "Then, please edit & copy the files to the jasmin's server manually and then add the Interceptors to the jasmin's config collection")
                    logging.info(" ")

                if syncCurrentFirst is True:
                    # Sync current data to Jasmin before waiting for changes
                    logging.info("Sync current configuration first is ENABLED")
                    self.pullAllConfigurations()
                    logging.info("")
                else:
                    logging.info(
                        "Sync current configuration first is DISABLED")
                    logging.info(
                        "Skipping synchronizing current configurations")
                    logging.info("")

                logging.info("Starting MongoDB cluster's Change Stream")
                logging.info("")
                for change in stream:
                    module = change["ns"]["coll"]
                    sub_id = change["documentKey"]["_id"]
                    change_type = change["operationType"]
                    if change_type == "invalidate":
                        return
                    if change_type in ["create", "drop", "dropDatabase", "rename", "modify"]:
                        logging.warn(
                            "Received a strange structure change. Ignoring stream content!!!!")
                        continue
                    if "updateDescription" in change and module in ["user", "smppccm"]:
                        sub_data = change["updateDescription"]["updatedFields"]
                    elif "fullDocument" in change:
                        sub_data = change["fullDocument"]
                    else:
                        sub_data = {}

                    if self.get_bill_managment_state() is True and module == "user":
                        if isinstance(sub_data, dict) and "mt_messaging_cred quota balance" in sub_data:
                            del sub_data["mt_messaging_cred quota balance"]
                        if isinstance(sub_data, dict) and "mt_messaging_cred quota sms_count" in sub_data:
                            del sub_data["mt_messaging_cred quota sms_count"]

                    if isinstance(sub_data, dict) and "_id" in sub_data:
                        del sub_data["_id"]

                    if not change_type == "delete" and len(sub_data.keys()) < 1:
                        continue

                    logging.info(
                        "Recieved notice of change in configurations from MongoDB cluster. Applies to:")
                    self.stream_handler(
                        payload={
                            "module": module,
                            "sub_id": sub_id,
                            "change_type": change_type,
                            "sub_data": sub_data
                        }
                    )
                    logging.info("Finished applying configurations")

        except MongoErrors.PyMongoError as err:
            # The ChangeStream encountered an unrecoverable error or the
            # resume attempt failed to recreate the cursor.
            logging.error(err)
            pass
        except KeyboardInterrupt:
            pass

    def stream_handler(self, payload):
        """Handle callback for jasmin stream"""
        jasmin_proxy = JasminTelnetProxy(
            host=self.cli_conn.get("host", DEFAULT_CLI_HOST),
            port=self.cli_conn.get("port", DEFAULT_CLI_PORT),
            timeout=self.cli_conn.get("timeout", DEFAULT_CLI_TIMEOUT),
            auth=self.cli_conn.get("auth", DEFAULT_CLI_AUTH),
            username=self.cli_conn.get("username", DEFAULT_CLI_USERNAME),
            password=self.cli_conn.get("password", DEFAULT_CLI_PASSWORD),
            standard_prompt=self.cli_conn.get(
                "standard_prompt", DEFAULT_CLI_STANDARD_PROMPT),
            interactive_prompt=self.cli_conn.get(
                "interactive_prompt", DEFAULT_CLI_INTERACTIVE_PROMPT),
            log_status=True,
            logger=self.logger_callback
        )

        if payload["change_type"] == "sync":
            """ Sync """
            jasmin_proxy.sync(
                module=payload["module"],
                sub_modules_data=payload["sub_data"]
            )

        if payload["change_type"] == "insert":
            """ Add """
            jasmin_proxy.add(
                module=payload["module"],
                sub_id=payload["sub_id"],
                options=payload["sub_data"]
            )

        if payload["change_type"] in ["update", "replace"]:
            """ Edit """
            jasmin_proxy.edit(
                module=payload["module"],
                sub_id=payload["sub_id"],
                options=payload["sub_data"]
            )

        if payload["change_type"] == "delete":
            """ Remove """
            jasmin_proxy.remove(
                module=payload["module"],
                sub_id=payload["sub_id"]
            )



import argparse
import logging
import os
import sys

import pkg_resources

from jasmin_mongo_configuration.defaults import *
from jasmin_mongo_configuration.mongodb import MongoDB


class ConfigurationStreamer:
    def __init__(
        self,
        mongo_connection_string: str,
        configuration_database: str,
        sync_current_first: bool = DEFAULT_SYNC_CURRENT_FIRST,
        bill_managment: bool = DEFAULT_BILL_MANAGMENT,
        cli_host: str = DEFAULT_CLI_HOST,
        cli_port: int = DEFAULT_CLI_PORT,
        cli_timeout: int = DEFAULT_CLI_TIMEOUT,
        cli_auth: bool = DEFAULT_CLI_AUTH,
        cli_username: str = DEFAULT_CLI_USERNAME,
        cli_password: str = DEFAULT_CLI_PASSWORD,
        cli_standard_prompt: str = DEFAULT_CLI_STANDARD_PROMPT,
        cli_interactive_prompt: str = DEFAULT_CLI_INTERACTIVE_PROMPT,
        logPath: str = DEFAULT_LOG_PATH,
        logLevel: str = DEFAULT_LOG_LEVEL
    ):

        logFormatter = logging.Formatter(
            "%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")
        rootLogger = logging.getLogger()
        rootLogger.setLevel(logLevel)

        consoleHandler = logging.StreamHandler()
        consoleHandler.setLevel(logLevel)
        consoleHandler.setFormatter(logFormatter)
        rootLogger.addHandler(consoleHandler)

        if not os.path.exists(logPath):
            os.makedirs(logPath)

        fileHandler = logging.FileHandler(
            f"{logPath.rstrip('/')}/jasmin_mongo_configuration.log")
        fileHandler.setLevel(logLevel)
        fileHandler.setFormatter(logFormatter)
        rootLogger.addHandler(fileHandler)

        logging.info("*********************************************")
        logging.info("::Jasmin MongoDB Configuration::")
        logging.info("")

        mongosource = MongoDB(
            connection_string=mongo_connection_string,
            database_name=configuration_database
        )
        if mongosource.startConnection() is True:
            mongosource.set_bill_managment_state(bill_managment)
            mongosource.stream(
                cli_host=cli_host,
                cli_port=cli_port,
                cli_timeout=cli_timeout,
                cli_auth=cli_auth,
                cli_username=cli_username,
                cli_password=cli_password,
                cli_standard_prompt=cli_standard_prompt,
                cli_interactive_prompt=cli_interactive_prompt,
                syncCurrentFirst=sync_current_first
            )


def str2bool(v: str) -> bool:
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'y', 'true', 't', '1'):
        return True
    elif v.lower() in ('no', 'n', 'false', 'f', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')

def printInterceptorFromCLI(interceptor:str):
    print(f"{pkg_resources.resource_string(__name__, f'interceptors/{interceptor}.py').decode('utf8')}", flush=True)

def interceptMTFromCLI():
    import jasmin_mongo_configuration.interceptors.mt

def interceptMOFromCLI():
    import jasmin_mongo_configuration.interceptors.mo

def startFromCLI():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
                                     description=f"Jasmin MongoDB Configuration, Links Jasmin SMS Gateway to MongoDB cluster's Change Stream (can be one node).")

    parser.add_argument('-v', '--version',
                        action='version',
                        version=f'%(prog)s {pkg_resources.get_distribution("jasmin_mongo_configuration").version}',
                        help="\n".join([
                            'Show version',
                            ' '
                        ]))

    mongoConfigurationsParserGroup = parser.add_argument_group(
        title='MongoDB',
        description="\n".join([
            'MongoDB cluster (can be one node) connection configuration section. You can use environment variables instead of command line arguments.',
        ]))
    mongoConfigurationsParserGroup.add_argument('-c',
                                                type=str,
                                                dest='mongo_connection_string',
                                                metavar='$connection_string',
                                                required=os.getenv(
                                                    "MONGO_CONNECTION_STRING") is None,
                                                default=os.getenv(
                                                    "MONGO_CONNECTION_STRING"),
                                                help="\n".join([
                                                    'MongoDB Connection String (Default: ** Required **)',
                                                    'Alternatively: You can use environment variable MONGO_CONNECTION_STRING',
                                                    'Example: mongodb://mongoroot:mongopass@mongodb1:27017,mongodb2:27017,mongodb3:27017/?authSource=admin&replicaSet=rs',
                                                    'MongoDB connection string to connect to the cluster',
                                                    ' '
                                                ]))

    mongoConfigurationsParserGroup.add_argument('-db',
                                                type=str,
                                                dest='configuration_database',
                                                metavar='$database_name',
                                                required=os.getenv(
                                                    "MONGO_CONFIGURATION_DATABASE") is None,
                                                default=os.getenv(
                                                    "MONGO_CONFIGURATION_DATABASE"),
                                                help="\n".join([
                                                    'Configuration Database (Default: ** Required **)',
                                                    'Alternatively: You can use environment variable MONGO_CONFIGURATION_DATABASE',
                                                    'MongoDB database name where you have saved the jasmin configurations',
                                                    ' '
                                                ]))

    appConfigurationsParserGroup = parser.add_argument_group(
        title='Features',
        description="\n".join([
            'Application features section. You can enable or disable them. You can use environment variables instead of command line arguments.',
        ]))
    appConfigurationsParserGroup.add_argument('-sync',
                                              type=str2bool,
                                              dest='sync_current_first',
                                              metavar='$is_start_sync',
                                              required=False,
                                              default=bool(os.getenv("SYNC_CURRENT_FIRST", 'yes' if DEFAULT_SYNC_CURRENT_FIRST else 'no').lower(
                                              ) in ['yes', 'true', 't', 'y', '1']),
                                              help="\n".join([
                                                  f'Sync current configuration first (Default: {"Enabled" if DEFAULT_SYNC_CURRENT_FIRST else "Disabled"})',
                                                  'Options: [Enable (true, t, yes, y, 1) and Disable (false, f, no, n, 0)]',
                                                  'Alternatively: You can use environment variable SYNC_CURRENT_FIRST'
                                                  'When enabled, will sync the current configurations first before monitoring for any changes',
                                                  ' '
                                              ]))

    appConfigurationsParserGroup.add_argument('-bill',
                                              type=str2bool,
                                              dest='bill_managment',
                                              metavar='$is_bill_managment',
                                              required=False,
                                              default=bool(os.getenv("JASMIN_MONGO_CONFIGURATION_BILL_MANAGMENT",
                                                                     'yes' if DEFAULT_BILL_MANAGMENT else 'no').lower() in ['yes', 'y']),
                                              help="\n".join([
                                                  f'Bill Managment (Default: {"Enabled" if DEFAULT_BILL_MANAGMENT else "Disabled"})',
                                                  'Options: [Enable (true, t, yes, y, 1) and Disable (false, f, no, n, 0)]',
                                                  'Alternatively: You can use environment variable JASMIN_MONGO_CONFIGURATION_BILL_MANAGMENT',
                                                  'When enabled, the "mt_messaging_cred quota balance" and "mt_messaging_cred quota sms_count" fields in the `user` module',
                                                  'will be ignored by the Change Stream. Then a special mt/mo interceptors will be installed in jasmin to check for billable messages',
                                                  'and will be used to bill users.',
                                                  ' '
                                              ]))

    appConfigurationsParserGroup.add_argument('-get-interceptor',
                                              type=str,
                                              choices=['mo', 'mt'],
                                              required=False,
                                              help="\n".join([
                                                  'Print the billing manager MO or MT interceptor and exit',
                                                  ' '
                                              ]))

    jasminParserGroup = parser.add_argument_group(
        title='Jasmin Connection',
        description="\n".join([
            'Jasmin Connection Configurations. You can use the environment variables to set the values instead of command line arguments.',
        ]))
    jasminParserGroup.add_argument('-H',
                                   type=str,
                                   dest='cli_host',
                                   metavar='$host',
                                   required=False,
                                   default=os.getenv(
                                       "JASMIN_CLI_HOST", DEFAULT_CLI_HOST),
                                   help="\n".join([
                                       f'Jasmin Host (Default: "%(default)s")',
                                       'Alternatively: You can use environment variable JASMIN_CLI_HOST',
                                       'The hostname of the jasmin server',
                                       ' '
                                   ]))

    jasminParserGroup.add_argument('-P',
                                   type=int,
                                   dest='cli_port',
                                   metavar='$port',
                                   required=False,
                                   default=int(
                                       os.getenv("JASMIN_CLI_PORT", DEFAULT_CLI_PORT)),
                                   help="\n".join([
                                       f'Jasmin CLI Port (Default: "%(default)s")',
                                       'Alternatively: You can use environment variable JASMIN_CLI_PORT',
                                       'The port of the jasmin server cli',
                                       ' '
                                   ]))

    jasminParserGroup.add_argument('-t',
                                   type=int,
                                   dest='cli_timeout',
                                   metavar='$timeout',
                                   required=False,
                                   default=int(
                                       os.getenv("JASMIN_CLI_TIMEOUT", DEFAULT_CLI_TIMEOUT)),
                                   help="\n".join([
                                       f'Jasmin CLI Timeout (Default: "%(default)s")',
                                       'Alternatively: You can use environment variable JASMIN_CLI_TIMEOUT',
                                       'The timeout for the CLI connection. Should be increased if your network is not stable.',
                                       ' '
                                   ]))

    jasminParserGroup.add_argument('-standard-prompt',
                                   type=str,
                                   dest='cli_standard_prompt',
                                   metavar='$standard_prompt',
                                   required=False,
                                   default=os.getenv(
                                       "JASMIN_CLI_STANDARD_PROMPT", DEFAULT_CLI_STANDARD_PROMPT),
                                   help="\n".join([
                                       f'Jasmin CLI Standard Prompt (Default: "%(default)s")',
                                       'Alternatively: You can use environment variable JASMIN_CLI_STANDARD_PROMPT',
                                       'There shouldn\'t be a need to change this.',
                                       ' '
                                   ]))

    jasminParserGroup.add_argument('-interactive-prompt',
                                   type=str,
                                   dest='cli_interactive_prompt',
                                   metavar='$interactive_prompt',
                                   required=False,
                                   default=os.getenv(
                                       "JASMIN_CLI_INTERACTIVE_PROMPT", DEFAULT_CLI_INTERACTIVE_PROMPT),
                                   help="\n".join([
                                       f'Jasmin CLI Interactive Prompt (Default: "%(default)s")',
                                       'Alternatively: You can use environment variable JASMIN_CLI_INTERACTIVE_PROMPT',
                                       'There shouldn\'t be a need to change this.',
                                       ' '
                                   ]))

    jasminParserGroup.add_argument('-auth',
                                   type=str2bool,
                                   dest='cli_auth',
                                   metavar='$is_auth',
                                   required=False,
                                   default=bool(os.getenv(
                                       "JASMIN_CLI_AUTH", 'yes' if DEFAULT_CLI_AUTH else 'no').lower() in ['yes', 'y']),
                                   help="\n".join([
                                       f'Jasmin CLI Auth (Default: {"Enabled" if DEFAULT_CLI_AUTH else "Disabled"})',
                                       'Options: [Enable (true, t, yes, y, 1) and Disable (false, f, no, n, 0)]',
                                       'Alternatively: You can use environment variable JASMIN_CLI_AUTH',
                                       'When enabled, will use authentication for the telnet connection.',
                                       ' '
                                   ]))

    jasminParserGroup.add_argument('-u',
                                   type=str,
                                   dest='cli_username',
                                   metavar='$username',
                                   required=False,
                                   default=os.getenv(
                                       "JASMIN_CLI_USERNAME", DEFAULT_CLI_USERNAME),
                                   help="\n".join([
                                       f'Jasmin CLI Username (Default: "%(default)s")',
                                       'Alternatively: You can use environment variable JASMIN_CLI_USERNAME',
                                       'The jasmin telnet cli username',
                                       ' '
                                   ]))

    jasminParserGroup.add_argument('-p',
                                   type=str,
                                   dest='cli_password',
                                   metavar='$password',
                                   required=False,
                                   default=os.getenv(
                                       "JASMIN_CLI_PASSWORD", DEFAULT_CLI_PASSWORD),
                                   help="\n".join([
                                       f'Jasmin CLI Password (Default: "%(default)s")',
                                       'Alternatively: You can use environment variable JASMIN_CLI_PASSWORD',
                                       'The jasmin telnet cli password',
                                       ' '
                                   ]))

    loggingConfigsParserGroup = parser.add_argument_group(
        title='Logging',
        description="\n".join([
            'Logging Configurations. You can use the environment variables to set the values instead of command line arguments.',
        ]))
    loggingConfigsParserGroup.add_argument('--log-path',
                                           type=str,
                                           dest='logPath',
                                           metavar='$path',
                                           required=False,
                                           default=os.getenv(
                                               "JASMIN_MONGO_CONFIGURATION_LOG_PATH", DEFAULT_LOG_PATH),
                                           help="\n".join([
                                               f'Log Path (Default: "%(default)s")',
                                               'Alternatively: You can use environment variable JASMIN_MONGO_CONFIGURATION_LOG_PATH',
                                               ' '
                                           ]))

    loggingConfigsParserGroup.add_argument('--log-level',
                                           type=str,
                                           dest='logLevel',
                                           metavar='$level',
                                           required=False,
                                           default=os.getenv(
                                               "JASMIN_MONGO_CONFIGURATION_LOG_LEVEL", DEFAULT_LOG_LEVEL),
                                           help="\n".join([
                                               f'Log Level (Default: "%(default)s")',
                                               'Alternatively: You can use environment variable JASMIN_MONGO_CONFIGURATION_LOG_LEVEL',
                                               ' '
                                           ]))

    parserNamespace = parser.parse_args()

    if parserNamespace.get_interceptor is not None:
        printInterceptorFromCLI(parserNamespace.get_interceptor)
        sys.exit(0)

    if parserNamespace.get_interceptor is None:
        del parserNamespace.get_interceptor
        ConfigurationStreamer(**vars(parserNamespace))



"""
Jasmin MongoDB Configuration - MT Interceptor

This interceptor is used to check if a user have enough credit to send a message

Requirements:
    - pymongo: pip install pymongo
"""
import os

from pymongo import MongoClient
from pymongo import errors as MongoErrors
from pymongo.collection import Collection as MongoCollection
from pymongo.database import Database as MongoDatabase

# Get MongoDB connection parameters
MONGO_CONNECTION_STRING = os.getenv(
    'MONGO_CONNECTION_STRING', 'mongodb://{$MONGODB_USERNAME}:{$MONGODB_PASSWORD}@mongodb1:27017,mongodb2:27017,mongodb3:27017/?authSource=admin&replicaSet=rs')
MONGO_BILLING_DATABASE = os.getenv(
    'MONGO_BILLING_DATABASE', 'configs')
MONGO_BILLING_COLLECTION = os.getenv(
    'MONGO_BILLING_COLLECTION', 'user')
MONGO_BILLING_FIELD = os.getenv(
    'MONGO_BILLING_FIELD', 'mt_messaging_cred quota sms_count')


def charge_MT(user_id, amount):
    """Charge a user for a message

    :param user_id: The user id
    :type user_id: str
    :param amount: The amount to charge
    :type amount: int
    :return: The SMPP status
    :rtype: int
    """
    try:
        mongoclient = MongoClient(MONGO_CONNECTION_STRING)
        server_info = mongoclient.server_info()
        if isinstance(server_info, dict) and 'ok' in server_info and server_info['ok'] == 1:
            """Connected to MongoDB"""
            database: MongoDatabase = mongoclient[MONGO_BILLING_DATABASE]
            database_info = database.command("buildinfo")
            if isinstance(database_info, dict) and 'ok' in database_info and database_info['ok'] == 1:
                """database exists"""
                collection: MongoCollection = database[MONGO_BILLING_COLLECTION]
                updateresult = collection.update_one(
                    {"_id": user_id}, {'$inc': {f"{MONGO_BILLING_FIELD}": amount}})

                if updateresult.modified_count == 1:
                    """User has been charged"""
                    # Return ESME_ROK
                    return 0
                else:
                    """User has not been charged"""
                    # Return ESME_RTHROTTLED
                    return 88
            else:
                """database does not exist"""
                # Return ESME_RDELIVERYFAILURE
                return 254
        else:
            """Could not connect to MongoDB"""
            # Return ESME_RDELIVERYFAILURE
            return 254

    except Exception:
        # We got an error when calling for charging
        # Return ESME_RDELIVERYFAILURE
        return 254

    except MongoErrors:
        # We got an error when calling for charging
        # Return ESME_RDELIVERYFAILURE
        return 254


_pdu = routable.pdu
amount = -1
while hasattr(_pdu, 'nextPdu'):
    _pdu = _pdu.nextPdu
    amount -= 1

if 'source_add' not in routable.pdu.params:
    smpp_status = charge_MT(routable.user.uid, amount)
